/*
 *  $Id: nxrecv.c,v 1.6 1993/11/12 11:54:18 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: nxrecv.c,v 1.6 1993/11/12 11:54:18 doss Exp $";
#endif /* lint */

#include "mpid.h"

/* static */ int MPID_NX_debugrecv = 0;

/* 
  This file contains the routines to handle receiving a message

  Because we don't know the length of messages (at least the long ones with
  tag MPID_PT2PT2_TAG(src) tags), we never post a receive.  Rather, we have
  a MPID_NX_check_incoming routine that looks for headers.  Note that
  messages sent from a source to destination with the MPID_PT2PT2_TAG(src)
  are ordered (we assume that the message-passing system preserves order).
 */


/*
   Post a receive.

   Since we let the NX layer handle this, there isn't much to do.
   We DO handle the case of posting a BLOCKING receive by doing BLOCKING
   waits on incoming messages.

   Otherwise, we simply try to handle any receives that are ready for
   processing.
 */
int MPID_NX_post_recv( dmpi_recv_handle, is_available ) 
MPIR_RHANDLE *dmpi_recv_handle;
int          *is_available;
{
MPID_RHANDLE *mpid_recv_handle;
MPIR_RHANDLE *dmpi_unexpected;
int          found;

mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;
*is_available    = 0;

if (MPID_NX_debugrecv) {
    printf( "[%d] starting recv for tag = %d and source = %d  handle %08X\n", 
	    _NX_me, dmpi_recv_handle->tag, dmpi_recv_handle->source,
	    dmpi_recv_handle );
    }
/* At this time, we check to see if the message has already been received */
MPIR_search_unexpected_queue( dmpi_recv_handle->source, 
		   dmpi_recv_handle->tag, dmpi_recv_handle->contextid, 
		   &found, 1, &dmpi_unexpected );
if (found) {
    if (MPID_NX_debugrecv) {
	printf( "[%d] Found message in unexpected queue %08X\n", 
	        _NX_me, dmpi_unexpected );
	}
    /* Copy relevant data to recv_handle */
    mpid_recv_handle           = &dmpi_unexpected->dev_rhandle;
    dmpi_recv_handle->source   = dmpi_unexpected->source;
    dmpi_recv_handle->tag      = dmpi_unexpected->tag;
    dmpi_recv_handle->totallen = mpid_recv_handle->bytes_as_contig;
    /* *dmpi_recv_handle = *dmpi_unexpected; */
    dmpi_recv_handle->dev_rhandle.is_posted = 0;
    /* Need to free unexpected */
    /* We don't do any of the transfer; we'll wait until the complete_recv
       is called */
    if (MPID_NX_debugrecv) {
	printf( "[%d] Found message in temp area...\n", _NX_me );
	}
    DMPI_put_totallen( dmpi_recv_handle, mpid_recv_handle->bytes_as_contig );
    DMPI_put_from_contig( dmpi_recv_handle, (void *)(mpid_recv_handle->temp),
   	                  mpid_recv_handle->bytes_as_contig );
    FREE( mpid_recv_handle->temp );

    /* Now, switch to the dev handle in the previously allocated handle */
    mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;

    mpid_recv_handle->temp = 0;      /* In case of a cancel */
    mpid_recv_handle->done = YES;
    DMPI_mark_recv_completed(dmpi_recv_handle);

    *is_available = 1;
    return MPI_OK;
    }

/* If we got here, the message is not yet available */
if (MPID_NX_debugrecv) {
    printf( "[%d] Queue posted recv %08X\n", _NX_me, dmpi_recv_handle );
    }
/* Add to the posted receive queue */
MPIR_enqueue( &posted_recvs, dmpi_recv_handle, MPIR_QRHANDLE );
dmpi_recv_handle->dev_rhandle.is_posted = 1;

MPID_NX_check_incoming( MPID_NOTBLOCKING );

return MPI_OK;
}

/* Check for incoming messages.
    Input Parameter:
.   is_blocking - true if this routine should block until a message is
    available

    Returns -1 if nonblocking and no messages pending
 */    
static MPID_PACKET	recv_pkt;	
static int		recv_mid = -1;

int MPID_NX_check_incoming( is_blocking )
MPID_BLOCKING_TYPE is_blocking;
{
int          from;
char         *address;
MPIR_RHANDLE *dmpi_recv_handle;
int          actual_len;
MPID_RHANDLE *mpid_recv_handle;
int          is_posted;

if (recv_mid == -1) {
    recv_mid = irecv( MPID_PT2PT_TAG, &recv_pkt, sizeof(MPID_PACKET) );
    }
/* If nonblocking and no headers available, exit */
if (is_blocking == MPID_NOTBLOCKING) {
    if (!msgdone( recv_mid )) return -1;
    } else {
    msgwait( recv_mid );
    }
recv_mid = -1;

from = infonode();
/* We should check the size here for internal errors .... */

/* Is the message expected or not? */
DMPI_msg_arrived( from, recv_pkt.tag, recv_pkt.context_id, 
                  &dmpi_recv_handle, &is_posted );
mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;
if (is_posted) {
    DMPI_put_totallen( dmpi_recv_handle, recv_pkt.len );
    DMPI_put_from_contig( dmpi_recv_handle, (void *)(recv_pkt.buffer),
   	                  MPID_MIN(recv_pkt.len,MPID_PACKET_SIZE) );
    if (MPID_NX_debugrecv) {
	printf( "[%d] Got data from packet for %08X\n", _NX_me, dmpi_recv_handle );
	}
    /* Handle long messages... */
    if (recv_pkt.len > MPID_PACKET_SIZE) {
	if (MPID_NX_debugrecv) {
	    printf( "[%d] Receiving a long message from %d\n", _NX_me, from );
	    }
        /* Question: who is responsible for this allocation? */
	DMPI_put_into_contig( dmpi_recv_handle, (void **)(&address), 
			      recv_pkt.len - MPID_PACKET_SIZE, &actual_len );
        crecv( MPID_PT2PT2_TAG(from), address, recv_pkt.len-MPID_PACKET_SIZE );
        }
    DMPI_mark_recv_completed(dmpi_recv_handle);
    mpid_recv_handle->done = YES;
    }
else {
    /* We buffer the message on the receiver.   There are, of course, other
       strategies */
    address = (char *)MALLOC( recv_pkt.len ); 
    if (MPID_NX_debugrecv) {
	printf( "[%d] Save data from packet at %08x for %08X\n",
		_NX_me , address, dmpi_recv_handle);
	}
    if (!address) return MPI_EXAUSTED;
    memcpy( address, recv_pkt.buffer, MPID_MIN(recv_pkt.len,MPID_PACKET_SIZE) );
    if (recv_pkt.len > MPID_PACKET_SIZE) {
	/* Message has "extension" part */
        crecv( MPID_PT2PT2_TAG(from), address + MPID_PACKET_SIZE, 
		 recv_pkt.len-MPID_PACKET_SIZE );
	}
    mpid_recv_handle->temp            = address;
    mpid_recv_handle->bytes_as_contig = recv_pkt.len;
    }
recv_mid = irecv( MPID_PT2PT_TAG, &recv_pkt, sizeof(MPID_PACKET) );
return MPI_OK;    
}


/*
    This routine completes a particular receive.  It does this by processing
    incoming messages until the indicated message is received
 */
int MPID_NX_complete_recv( dmpi_recv_handle ) 
MPIR_RHANDLE *dmpi_recv_handle;
{
MPID_RHANDLE *mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;

if (MPID_NX_debugrecv) {
printf( "[%d] Complete recv %08X\n",
	_NX_me , dmpi_recv_handle);
}
while (!mpid_recv_handle->done) {
    (void)MPID_NX_check_incoming( MPID_BLOCKING );
    }
return MPI_OK;
}
