/*
 *  $Id: gather.c,v 1.9 1994/01/27 16:10:10 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: gather.c,v 1.9 1994/01/27 16:10:10 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Gather - 

@*/
int MPI_Gather ( sendbuf, sendcnt, sendtype, recvbuf, recvcnt, recvtype, 
		 root, comm )
void             *sendbuf;
int               sendcnt;
MPI_Datatype      sendtype;
void             *recvbuf;
int               recvcnt;
MPI_Datatype      recvtype;
int               root;
MPI_Comm          comm;
{
  MPI_Status status;
  int        size, rank;
  int        errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( ( (comm            == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL) ) ||
       ( (root            <  0)           && (errno = MPI_ERR_ROOT) )      || 
       ( (comm->comm_type == MPIR_INTER)  && (errno = MPI_ERR_COMM_INTER) ) )
    return (errno); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( (root >= size) && (errno = MPI_ERR_ROOT) )
    return (errno);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* If rank == root, then I recv lots, otherwise I send */
  if ( rank == root ) {
    int i, extent;
	MPI_Comm_request req;
	MPI_Status       status;

    MPI_Isend(sendbuf, sendcnt, sendtype, root, MPIR_GATHER_TAG, comm, &req);
    MPI_Type_extent(recvtype, &extent);
    for ( i=0; i<size; i++ ) {
      MPI_Recv( (void *)((int)recvbuf+i*extent*recvcnt), recvcnt, recvtype, i, 
		 MPIR_GATHER_TAG, comm, &status);
    }
	MPI_Wait(&req, &status);
  }
  else 
    MPI_Send(sendbuf, sendcnt, sendtype, root, MPIR_GATHER_TAG, comm);
  
  return (errno);
}

