/*
 *  $Id: attr_util.c,v 1.7 1994/02/02 22:29:17 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPIR_Attr_copy_node -

@*/
int MPIR_Attr_copy_node ( comm, comm_new, node )
MPI_Comm comm, comm_new;
MPIR_HBT_node *node;
{
  MPIR_Attr_key *attr_key = (MPIR_Attr_key *)(node->keyval);
  if (attr_key->copy_fn != (int (*)())0)
    (void) (*(attr_key->copy_fn))(comm, comm_new, node->keyval, attr_key->extra_state);

  return (MPI_SUCCESS);
}

/*+

MPIR_Attr_copy_subtree -

+*/
int MPIR_Attr_copy_subtree ( comm, comm_new, tree, subtree )
MPI_Comm comm, comm_new;
MPIR_HBT *tree;
MPIR_HBT_node *subtree;
{
  if(subtree != (MPIR_HBT_node *)0) {
    (void) MPIR_Attr_copy_node ( comm, comm_new, subtree );
    (void) MPIR_Attr_copy_subtree ( comm, comm_new, tree, subtree -> left );
    (void) MPIR_Attr_copy_subtree ( comm, comm_new, tree, subtree -> right );
  }
  return (MPI_SUCCESS);
}

/*+

MPIR_Attr_copy - copy a tree of attributes 

+*/
int MPIR_Attr_copy ( comm, comm_new )
MPI_Comm comm, comm_new;
{
  (void) MPIR_HBT_new_tree ( &(comm_new->attr_cache) );
  if ( comm_new->attr_cache != (MPIR_HBT *)0 )
    (void) MPIR_Attr_copy_subtree ( comm, comm_new, comm_new->attr_cache,
                                    comm->attr_cache->root );
  return (MPI_SUCCESS);
}


/*+

MPIR_Attr_free_node -

+*/
int MPIR_Attr_free_node ( comm, node )
MPI_Comm comm;
MPIR_HBT_node *node;
{
  MPIR_Attr_key *attr_key = (MPIR_Attr_key *)(node->keyval);

  if ( (node != (MPIR_HBT_node *)0) && (attr_key != 0) )
    if ( attr_key->delete_fn != (int (*)())0 ) 
      (void ) (*(attr_key->delete_fn))(comm, node->keyval, attr_key->extra_state);
  return (MPI_SUCCESS);
}

/*+

MPIR_Attr_free_subtree -

+*/
int MPIR_Attr_free_subtree ( comm, subtree )
MPI_Comm comm;
MPIR_HBT_node *subtree;
{
  if(subtree != (MPIR_HBT_node *)0) {
    (void) MPIR_Attr_free_subtree ( comm, subtree -> left );
    (void) MPIR_Attr_free_subtree ( comm, subtree -> right );
    (void) MPIR_Attr_free_node ( comm, subtree );
  }
  return (MPI_SUCCESS);
}

/*+

MPIR_Attr_free_tree -

+*/
int MPIR_Attr_free_tree ( comm )
MPI_Comm comm;
{
  if ( comm != (MPI_Comm)0 ) 
    if ( comm->attr_cache->root != (MPIR_HBT_node *)0 )
      (void) MPIR_Attr_free_subtree ( comm, comm->attr_cache->root );
  (void) MPIR_HBT_free_tree ( comm->attr_cache );
  return (MPI_SUCCESS);
}
