/*
 *  $Id: comm_dup.c,v 1.8 1994/02/02 21:20:20 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Comm_dup - duplicates an existing communicator with all its cached
               information

@*/
int MPI_Comm_dup ( comm, comm_out )
MPI_Comm comm, *comm_out;
{
  MPI_Comm new_comm;

  /* Check for non-null communicator */
  if ( comm == (MPI_Comm)0 ) {
    (*comm_out) = (MPI_Comm)0;
	return (MPI_ERR_COMM_NULL);
  }

  /* Only inter-communicators allowed! */
  if ( comm->comm_type != MPIR_INTRA ) {
	(*comm_out) = (MPI_Comm)0;
	return (MPI_ERR_COMM_INTER);
  }
	
  /* Duplicate communicator */
  new_comm                  = NEW(MPIR_COMMUNICATOR);
  new_comm->comm_type       = MPIR_INTRA;
  (void) MPIR_GROUP_dup ( comm->group, &(new_comm->group) );
  (void) MPIR_Context_alloc ( comm, 2, &(new_comm->send_context) );
  new_comm->recv_context    = new_comm->send_context;
  new_comm->comm_cache      = 0;
  new_comm->topology.type   = MPI_UNDEFINED;
  (void) MPIR_Attr_copy ( comm, new_comm ); 
  (void) MPIR_Comm_make_coll ( new_comm );
  (*comm_out)               = new_comm;
  return(MPI_SUCCESS);
}

