/*
 *  $Id: group_incl.c,v 1.5 1994/02/02 21:15:26 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Group_incl - 

@*/
int MPI_Group_incl ( group, n, ranks, group_out )
MPI_Group group, *group_out;
int       n, *ranks;
{
  int i, rank;
  MPI_Group new_group;

  /* Check for a NULL input group or NULL sized new group */
  if ( (group == (MPI_Group)0) || (n <= 0) ) {
    (*group_out) = (MPI_Group)0;
    return (MPI_SUCCESS);
  }
  
  /* Create the new group */
  new_group = (*group_out)  = NEW(MPIR_GROUP);
  new_group->ref_count      = 1;
  new_group->local_rank     = MPI_UNDEFINED;
  new_group->set_mark       = (int *)0;
  new_group->np             = n;
  new_group->lrank_to_grank = (int *) MALLOC( n * sizeof(int) );

  /* Fill in the lrank_to_grank list */
  for (i=0; i<n; i++) {
    if ( (rank = ranks[i]) < group->np )
      new_group->lrank_to_grank[i] = group->lrank_to_grank[rank];
    else
      new_group->lrank_to_grank[i] = MPI_UNDEFINED;
    if (group->local_rank == rank)
      new_group->local_rank = i;
  }

  /* Determine the previous and next powers of 2 */
  MPIR_Powers_of_2 ( new_group->np, &(new_group->N2_next), &(new_group->N2_prev) );

  return (MPI_SUCCESS);
}

