/*
 *  $Id: group_union.c,v 1.5 1994/02/02 21:19:31 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

#define MPIR_UNMARKED 0
#define MPIR_MARKED   1

/*@

MPI_Group_union - 

@*/
int MPI_Group_union ( group1, group2, group_out )
MPI_Group group1, group2, *group_out;
{
  int        i, j, global_rank;
  MPI_Group  new_group;
  int        n;

  /* Check for NULL groups */
  if ( (group1 == (MPI_Group)0) && (group2 == (MPI_Group)0) ) {
    (*group_out) = group1;
    return (MPI_SUCCESS);
  }
  if ( group1 == (MPI_Group)0 ) {
    (void) MPIR_GROUP_dup ( group2, group_out );
    return (MPI_SUCCESS);
  }
  if ( group2 == (MPI_Group)0 ) {
    (void) MPIR_GROUP_dup ( group2, group_out );
    return (MPI_SUCCESS);
  }
  
  /* Create the new group */
  new_group = (*group_out) = NEW(MPIR_GROUP);
  new_group->ref_count     = 1;
  new_group->local_rank    = group1->local_rank;
  new_group->set_mark      = (int *)0;
  
  /* Set the number in the union */
  n = group1->np + group2->np;

  /* Allocate set marking space for group2 if necessary */
  if (group2->set_mark == NULL)
    group2->set_mark = (int *) MALLOC( group2->np * sizeof(int) );

  /* Mark the union */
  for ( j=0; j<group2->np; j++ ) {
    group2->set_mark[j] = MPIR_MARKED;
    for ( i=0; i<group1->np; i++ ) 
      if ( group1->lrank_to_grank[i] == group2->lrank_to_grank[j] ) {
        group2->set_mark[j] = MPIR_UNMARKED;
        n--;
        break;
      }
  }
  
  /* Alloc the memory */
  new_group->np             = n;
  new_group->lrank_to_grank = (int *) MALLOC( n * sizeof(int) );
  
  /* Fill in the space */
  n = group1->np;
  memcpy(new_group->lrank_to_grank,group1->lrank_to_grank,n*sizeof(int));
  for ( j=0; j<group2->np; j++ ) 
    if ( (group2->set_mark[j]==MPIR_MARKED) && (n < new_group->np) ) 
      new_group->lrank_to_grank[n++] = group2->lrank_to_grank[j];
  
  /* Find the local rank only if local rank not defined in group 1 */
  if ( new_group->local_rank == MPI_UNDEFINED ) {
    MPID_myrank(&global_rank);
    for( i=group1->np; i<new_group->np; i++ )
      if ( global_rank == new_group->lrank_to_grank[i] ) {
        new_group->local_rank = i;
        break;
      }
  }

  /* Determine the previous and next powers of 2 */
  MPIR_Powers_of_2 ( new_group->np, &(new_group->N2_next), &(new_group->N2_prev) );

  return (MPI_SUCCESS);
}



