/*
 *  $Id: init.c,v 1.10 1994/02/02 21:17:49 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: init.c,v 1.10 1994/02/02 21:17:49 gropp Exp $";
#endif /* lint */
#include "mpi.h"
#include "mpisys.h"

#ifdef MPI_rs6000
#define mpir_init_fdtes_ mpir_init_fdtes
#define mpir_init_fcm_   mpir_init_fcm
#endif

/*@
   MPI_Init - Terminate MPI execution environment
@*/
int MPI_Init(argc,argv)
int  *argc;
char **argv;
{
    int size;

    /* initialize memory allocation data structures */
    MPIR_shandles   = SBinit( sizeof( MPIR_SHANDLE ), 100, 100 );
    MPIR_rhandles   = SBinit( sizeof( MPIR_RHANDLE ), 100, 100 );
    MPIR_dtes       = SBinit( sizeof( MPIR_DATATYPE ), 100, 100 );
    MPIR_qels       = SBinit( sizeof( MPIR_QEL ), 100, 100 );
    MPIR_fdtels     = SBinit( sizeof( MPIR_FDTEL ), 100, 100 );
    MPIR_hbts       = SBinit( sizeof( MPIR_HBT ), 5, 5 );
    MPIR_hbt_els    = SBinit( sizeof( MPIR_HBT_node ), 20, 20);

    /* set up pre-defined data types */
    MPIR_int_dte              = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_int_dte->dte_type    = MPIR_INT;
    MPIR_int_dte->is_contig   = YES;
    MPIR_int_dte->extent      = sizeof( int );
    MPIR_float_dte            = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_float_dte->dte_type  = MPIR_FLOAT;
    MPIR_float_dte->is_contig   = YES;
    MPIR_float_dte->extent    = sizeof( float );
    MPIR_double_dte           = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_double_dte->dte_type = MPIR_DOUBLE;
    MPIR_double_dte->is_contig   = YES;
    MPIR_double_dte->extent   = sizeof( double );
    MPIR_complex_dte          = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_complex_dte->dte_type= MPIR_COMPLEX;
    MPIR_complex_dte->is_contig   = YES;
    MPIR_complex_dte->extent  = 2 * sizeof( float );
    MPIR_long_dte             = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_long_dte->dte_type   = MPIR_LONG;
    MPIR_long_dte->is_contig   = YES;
    MPIR_long_dte->extent     = sizeof( long );
    MPIR_short_dte            = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_short_dte->dte_type  = MPIR_SHORT;
    MPIR_short_dte->is_contig   = YES;
    MPIR_short_dte->extent    = sizeof( short );
    MPIR_char_dte             = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_char_dte->extent     = sizeof( char );
    MPIR_char_dte->is_contig   = YES;
    MPIR_char_dte->dte_type   = MPIR_CHAR;
    MPIR_byte_dte             = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_byte_dte->extent     = sizeof( char );
    MPIR_byte_dte->dte_type   = MPIR_BYTE;
    MPIR_byte_dte->is_contig   = YES;
    MPIR_uchar_dte            = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_uchar_dte->extent    = sizeof( unsigned char );
    MPIR_uchar_dte->dte_type  = MPIR_UCHAR;
    MPIR_uchar_dte->is_contig   = YES;
    MPIR_ushort_dte           = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_ushort_dte->extent   = sizeof( unsigned short );
    MPIR_ushort_dte->dte_type = MPIR_USHORT;
    MPIR_ushort_dte->is_contig   = YES;
    MPIR_ulong_dte            = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_ulong_dte->extent    = sizeof( unsigned long );
    MPIR_ulong_dte->dte_type  = MPIR_ULONG;
    MPIR_ulong_dte->is_contig   = YES;
    MPIR_uint_dte             = (MPIR_DATATYPE *) SBalloc( MPIR_dtes );
    MPIR_uint_dte->extent     = sizeof( unsigned int );
    MPIR_uint_dte->dte_type   = MPIR_UINT;
    MPIR_uint_dte->is_contig   = YES;

    /* Set the values of the Fortran versions */
    /* Logical and character aren't portable in the code below */
    mpir_init_fdtes_( &MPIR_int_dte, &MPIR_float_dte, &MPIR_double_dte,
                     &MPIR_complex_dte, &MPIR_int_dte, &MPIR_char_dte, 
                     &MPIR_byte_dte );

    /* initialize queues */
    posted_sends.first        = posted_sends.last        = NULL;
    posted_sends.maxlen       = posted_sends.currlen     = 0;

    posted_recvs.first        = posted_recvs.last        = NULL;
    posted_recvs.maxlen       = posted_recvs.currlen     = 0; 

    unexpected_recvs.first    = unexpected_recvs.last    = NULL;
    unexpected_recvs.maxlen   = unexpected_recvs.currlen = 0;

    MPI_COMM_WORLD              = NEW(MPIR_COMMUNICATOR);    
    MPI_COMM_WORLD->comm_type   = MPIR_INTRA;
    MPID_mysize( &size );
    MPID_myrank( &MPIR_tid);
    MPI_COMM_WORLD->group       = MPIR_CreateGroup( size );
    MPIR_SetToIdentity( MPI_COMM_WORLD->group );

    MPI_COMM_WORLD->send_context = MPIR_PT2PT_CONTEXT;
    MPI_COMM_WORLD->recv_context = MPIR_PT2PT_CONTEXT;
    MPIR_Comm_make_coll ( MPI_COMM_WORLD );

    MPIR_HBT_new_tree ( &(MPI_COMM_WORLD->attr_cache) );

    /* Place-holder for cache */
    MPI_COMM_WORLD->comm_cache    = 0;

    mpir_init_fcm_( &MPI_COMM_WORLD );

    /* Search for "-mpi debug" options etc.  We need a better interface.... */
#ifdef DEVICE_CHAMELEON
    if (SYArgHasName( argc, argv, 1, "-chdebug" )) {
	MPID_SetSendDebugFlag( 1 );
	MPID_SetRecvDebugFlag( 1 );
	}
#endif
    /* barrier */

    return MPI_OK;
}
