/*
 *  $Id: start.c,v 1.6 1994/01/31 23:45:39 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/* NOTES 
   We mark all sends and receives as non-blocking because that is safe here;
   unfortunately, we don't have enough information in the current send/recv
   handle to determine if we are blocking or not.
 */
/*@
    MPI_Start - 
@*/
int MPI_Start( handle )
MPI_Comm_request handle;
{
    int is_available;

    if (handle->type == MPIR_SEND)
    {
#if !defined(WHEN_WE_DO_THIS)
	if (handle->shandle.datatype->is_contig) {
	    handle->shandle.dev_shandle.start = handle->shandle.bufadd;
            handle->shandle.dev_shandle.bytes_as_contig =
		handle->shandle.count * handle->shandle.datatype->extent;
	    }
#endif
	/* device will post the send */
        MPID_set_send_is_nonblocking( &(handle->shandle.dev_shandle), 1 );
	MPID_post_send( &(handle->shandle) );
    }
    else if (handle->type == MPIR_RECV)
    {
#if !defined(WHEN_WE_DO_THIS)
	if (handle->rhandle.datatype->is_contig) {
	    handle->rhandle.dev_rhandle.start = handle->rhandle.bufadd;
            handle->rhandle.dev_rhandle.bytes_as_contig =
		handle->rhandle.count * handle->rhandle.datatype->extent;
	    }
#endif
        /* device will handle queueing of MPIR recv handle in posted-recv queue         */
        MPID_set_recv_is_nonblocking( &(handle->rhandle.dev_rhandle), 1 );
	MPID_post_recv( &(handle->rhandle), &(is_available) );
    }
    else
	MPIR_error("bad handle type in MPI_START");
    return MPI_OK;
}


