/*
 *  $Id: graph_nbr.c,v 1.5 1994/01/25 21:58:02 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

#ifndef MPIR_MIN
#define MPIR_MIN(a,b) ((a)>(b)?(b):(a))
#endif

/*@

MPI_Graph_neighbors - returns the neighbors of a node associated 
                      with a graph topology

@*/
int MPI_Graph_neighbors ( comm, rank, maxneighbors, neighbors )
MPI_Comm  comm;
int               rank;
int               maxneighbors;
int              *neighbors;
{
  int i, begin, end;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL)) ||
      ((comm->topology.type != MPI_GRAPH)   && (errno = MPI_ERR_TOPOLOGY))  ||
      ((rank                <  0)           && (errno = MPI_ERR_RANK))      ||
      ((rank >= (comm->topology.tag.graph.nnodes))&&(errno = MPI_ERR_RANK)) ||
      ((maxneighbors        <= 0)           && (errno = MPI_ERR_UNKNOWN))   ||
      ((neighbors           == (int *)0)    && (errno = MPI_ERR_NULL))       )
    return (errno);

  /* Get neighbors */
  if ( rank == 0 ) 
    begin = 0;
  else
    begin = comm->topology.tag.graph.index[rank-1];
  end = comm->topology.tag.graph.index[rank];
  end = MPIR_MIN(end-begin, maxneighbors) + begin;
  for ( i=begin; i<end; i++ )
    neighbors[i-begin] = comm->topology.tag.graph.edges[i];

  return (errno);
}
